#!/usr/bin/sh
#
# sassist.sh
#
# Export sosreport to Dell PowerEdge Server iDRAC for out-of-band access.
#
# v0.8.3
#
# Usage:
#	Invoked by sassist.service when SupportAssist USB block device
#	is attached.
#
#
# Copyright 2017 Charles Rose, Dell Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.


# User configurable options
SCONFIG_PLUGINS="BOOT,BTRFS,CRASH,DISK,IB,ISCSI,LVM,MEM,MOD,MPIO,NET,SRAID,SYSCONFIG,SYSFS,UDEV,X"
SOSREPORT="/usr/sbin/sosreport"
SUPPORTCONFIG="/sbin/supportconfig"

#--------------------------------------------------------------
ipmi()
{
	OUT=$(/usr/sbin/ipmi-raw 0 30 a8 $@)
	if [ $? -eq 0 ]; then
		RET=$(printf "$OUT"| awk '{print $4}')
		if [ "$RET" -eq 0 ]; then
			return 0
		else
			return 1
		fi
	fi
	return 1
}

can_filter="ipmi 0 1 3 0 0"
cannot_filter="ipmi 0 1 1 0 0"
supported="ipmi 1 0 0"
end_full="ipmi 2 0 3"
end_partial="ipmi 2 1 3"
do_close="ipmi 2 2 0"
do_fail="ipmi 2 3 0"

SVCTAG=$(cat /sys/devices/virtual/dmi/id/product_serial)
TMP_DIR=$(mktemp -d)
OUTFILE_F="${TMP_DIR}/OSC-FR-Report-${SVCTAG}.zip"
# Partial Report - TODO
OUTFILE_P="${TMP_DIR}/OSC-PR-Report-${SVCTAG}.zip"

can_do_sassist()
{
	#TODO: until tools like soscleaner become in-distro
	$cannot_filter
	return $?
}

# Run sosreport and zip results
do_sosreport()
{
	$SOSREPORT --batch -a\
		--tmp-dir ${TMP_DIR} --build --quiet \
		--name ${SVCTAG} || return 1
	# Windows does not like some filenames
	find ${TMP_DIR} -name "modinfo_*" -execdir mv '{}' modinfo \;
	find ${TMP_DIR} -name "find_*" -execdir rm -r '{}' \;
	find ${TMP_DIR} -name "*:*" -execdir rm -rf '{}' \;
	find ${TMP_DIR} -type s -execdir rm -f '{}' \;

	$(cd ${TMP_DIR}/sosreport-* && zip -y -q -r ${OUTFILE_F} . )
	return $?
}

# Run supportconfig and zip results
do_supportconfig()
{
	$SUPPORTCONFIG -Q -d -k -t ${TMP_DIR} \
		-i ${SCONFIG_PLUGINS} -B ${SVCTAG} || return 1

	$(cd ${TMP_DIR}/nts_${SVCTAG} && zip -q -r ${OUTFILE_F} . )
	return $?
}

do_report()
{
	if $(findmnt | grep -q "$MEDIA_DIR") && ! $supported; then
		RETVAL=3
		do_stop
	fi

	if [ -x "$SOSREPORT" ]; then
		do_sosreport >/dev/null 2>&1
	elif [ -x "$SUPPORTCONFIG" ]; then
		do_supportconfig >/dev/null 2>&1
	else
		RETVAL=5
		do_stop
	fi

	if [ $? -ne 0 ]; then
		RETVAL=6
		do_stop
	fi

	SHA_F=$(sha256sum ${OUTFILE_F}| cut -d' ' -f1| sed 's/.\{2\}/& /g')

	cp -f ${TMP_DIR}/OSC-*zip ${MEDIA_DIR}/ >/dev/null 2>&1
	umount -r ${MEDIA_DIR}
	if [ $? -ne 0 ]; then
		RETVAL=1
		do_stop
	fi
	rm -rf ${TMP_DIR}

	# Close connection with checksum
	$end_full "${SHA_F}"
	$do_close
}

do_stop()
{
	$do_close
	$do_fail
	exit $RETVAL
}

# Main
case $1 in
	enable)
		can_do_sassist
		exit $?
	;;
	start)
		can_do_sassist && \
		do_report
		exit $?
	;;
	stop)
		RETVAL=0
		do_stop
	;;
	*)
		printf "Usage: %s <enable|start|stop>\n" $0
		exit 0
	;;
esac
